@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2013-2016 Horde LLC (http://www.horde.org/)
+ * Copyright 2013-2017 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
  *
  * @category  Horde
- * @copyright 2013-2016 Horde LLC
+ * @copyright 2013-2017 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Vfs
  */
@@ -16,7 +16,7 @@
  *
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2013-2016 Horde LLC
+ * @copyright 2013-2017 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Vfs
  */
@@ -200,6 +200,13 @@ class Horde_Vfs_Mongo extends Horde_Vfs_Base
                 break;
 
             case 'string':
+                // MONGO currently has no ability to stream data TO the
+                // server. I.e., there is no opposite version of
+                // MongoGridFSFile::getResource().
+                $data = $this->_ensureSeekable($data);
+                if (is_resource($data)) {
+                    $data = stream_get_contents($data);
+                }
                 $this->_files->storeBytes($data, $mdata);
                 break;
             }
