@@ -12,7 +12,7 @@
  * The table structure for the VFS can be created with the horde-db-migrate
  * script from the Horde_Db package.
  *
- * Copyright 2002-2016 Horde LLC (http://www.horde.org/)
+ * Copyright 2002-2017 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
@@ -207,24 +207,31 @@ class Horde_Vfs_Sql extends Horde_Vfs_Base
     {
         /* Don't need to check quota here since it will be checked when
          * writeData() is called. */
-        return $this->writeData($path,
-                                $name,
-                                file_get_contents($tmpFile),
-                                $autocreate);
+        if (!$stream = @fopen($tmpFile, 'rb')) {
+            throw new Horde_Vfs_Exception('Unable to open ' . $tmpFile);
+        }
+
+        $result = $this->writeData(
+            $path, $name, $stream, $autocreate);
+        fclose($stream);
+
+        return $result;
     }
 
     /**
      * Store a file in the VFS from raw data.
      *
-     * @param string $path         The path to store the file in.
-     * @param string $name         The filename to use.
-     * @param string $data         The file data.
-     * @param boolean $autocreate  Automatically create directories?
+     * @param string $path           The path to store the file in.
+     * @param string $name           The filename to use.
+     * @param string|resource $data  The data as a string or stream resource.
+     *                               Resources allowed  @since  2.4.0
+     * @param boolean $autocreate    Automatically create directories?
      *
      * @throws Horde_Vfs_Exception
      */
     public function writeData($path, $name, $data, $autocreate = false)
     {
+        $data = $this->_ensureSeekable($data);
         $this->_checkQuotaWrite('string', $data, $path, $name);
 
         $path = $this->_convertPath($path);
@@ -692,18 +699,23 @@ class Horde_Vfs_Sql extends Horde_Vfs_Base
         }
 
         $where = '';
+        $values = array();
         foreach ($criteria as $key => $value) {
             if (!empty($where)) {
                 $where .= ' AND ';
             }
-            $where .= $key . ' = ' . $this->_db->quote($value);
+            list($op, $val) = $this->_nullString($value);
+            $where .= $key . ' ' . $op;
+            $values = array_merge($values, $val);
         }
 
-        $sql = sprintf('SELECT %s FROM %s WHERE %s',
-                       $field, $table, $where);
+        $sql = sprintf(
+            'SELECT %s FROM %s WHERE %s',
+            $field, $table, $where
+        );
 
         try {
-            $result = $this->_db->selectValue($sql);
+            $result = $this->_db->selectValue($sql, $values);
             $columns = $this->_db->columns($table);
         } catch (Horde_Db_Exception $e) {
             throw new Horde_Vfs_Exception($e);
