@@ -2,7 +2,7 @@
 /**
  * Test the SMB based virtual file system.
  *
- * Copyright 2011-2016 Horde LLC (http://www.horde.org/)
+ * Copyright 2011-2017 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
@@ -202,6 +202,11 @@ class Horde_Vfs_SmbTest extends Horde_Vfs_TestBase
         $this->_listFolder();
     }
 
+    public function testNullRoot()
+    {
+        $this->_nullRoot();
+    }
+
     public function testHostspecWithPath()
     {
         self::$vfs->createFolder('', 'hostspectest');
